package com.sun.java.util.jar.pack;

import java.util.Iterator;
import java.util.jar.JarEntry;
import java.util.HashSet;
import java.util.zip.CheckedOutputStream;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.util.zip.CRC32;
import java.io.FileInputStream;
import java.io.File;
import java.io.IOException;
import java.util.jar.JarInputStream;
import java.io.BufferedInputStream;
import java.util.jar.JarOutputStream;
import java.io.InputStream;
import java.util.SortedMap;
import java.beans.PropertyChangeListener;
import java.util.jar.Pack200;

public class UnpackerImpl extends TLGlobals implements Pack200.Unpacker
{
    Object _nunp;
    
    public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        this.props.addListener(propertyChangeListener);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        this.props.removeListener(propertyChangeListener);
    }
    
    public UnpackerImpl() {
    }
    
    public SortedMap<String, String> properties() {
        return this.props;
    }
    
    public String toString() {
        return Utils.getVersionString();
    }
    
    public synchronized void unpack(InputStream in, JarOutputStream jarOutputStream) throws IOException {
        if (in == null) {
            throw new NullPointerException("null input");
        }
        if (jarOutputStream == null) {
            throw new NullPointerException("null output");
        }
        assert Utils.currentInstance.get() == null;
        int n = !this.props.getBoolean("com.sun.java.util.jar.pack.default.timezone");
        try {
            Utils.currentInstance.set(this);
            if (!(n == 0)) {
                Utils.changeDefaultTimeZoneToUtc();
            }
            int integer = this.props.getInteger("com.sun.java.util.jar.pack.verbose");
            BufferedInputStream in2 = new BufferedInputStream(in);
            if (Utils.isJarMagic(Utils.readMagic(in2))) {
                if (integer > 0) {
                    Utils.log.info("Copying unpacked JAR file...");
                }
                Utils.copyJarFile(new JarInputStream(in2), jarOutputStream);
            }
            else if (this.props.getBoolean("com.sun.java.util.jar.pack.disable.native")) {
                new UnpackerImpl.DoUnpack().run(in2, jarOutputStream);
                in2.close();
                Utils.markJarFile(jarOutputStream);
            }
            else {
                try {
                    new NativeUnpack(this).run(in2, jarOutputStream);
                }
                catch (UnsatisfiedLinkError | NoClassDefFoundError unsatisfiedLinkError | NoClassDefFoundError) {
                    new UnpackerImpl.DoUnpack().run(in2, jarOutputStream);
                }
                in2.close();
                Utils.markJarFile(jarOutputStream);
            }
        }
        finally {
            this._nunp = null;
            Utils.currentInstance.set(null);
            if (!(n == 0)) {
                Utils.restoreDefaultTimeZone();
            }
        }
        return;
    }
    
    public synchronized void unpack(File file, JarOutputStream jarOutputStream) throws IOException {
        if (file == null) {
            throw new NullPointerException("null input");
        }
        if (jarOutputStream == null) {
            throw new NullPointerException("null output");
        }
        try (final FileInputStream fileInputStream = new FileInputStream(file)) {
            this.unpack(fileInputStream, jarOutputStream);
        }
        if (this.props.getBoolean("com.sun.java.util.jar.pack.unpack.remove.packfile")) {
            file.delete();
        }
        return;
    }
    
    private class DoUnpack
    {
        final int verbose;
        final Package pkg;
        final boolean keepModtime;
        final boolean keepDeflateHint;
        final int modtime;
        final boolean deflateHint;
        final CRC32 crc;
        final ByteArrayOutputStream bufOut;
        final OutputStream crcOut;
        
        private DoUnpack() {
            this.verbose = UnpackerImpl.this.props.getInteger("com.sun.java.util.jar.pack.verbose");
            UnpackerImpl.this.props.setInteger("unpack.progress", 0);
            this.pkg = new Package();
            this.keepModtime = "keep".equals(UnpackerImpl.this.props.getProperty("com.sun.java.util.jar.pack.unpack.modification.time", "keep"));
            this.keepDeflateHint = "keep".equals(UnpackerImpl.this.props.getProperty("unpack.deflate.hint", "keep"));
            if (!this.keepModtime) {
                this.modtime = UnpackerImpl.this.props.getTime("com.sun.java.util.jar.pack.unpack.modification.time");
            }
            else {
                this.modtime = this.pkg.default_modtime;
            }
            this.deflateHint = (!this.keepDeflateHint && UnpackerImpl.this.props.getBoolean("unpack.deflate.hint"));
            this.crc = new CRC32();
            this.bufOut = new ByteArrayOutputStream();
            this.crcOut = new CheckedOutputStream(this.bufOut, this.crc);
        }
        
        public void run(BufferedInputStream bufferedInputStream, JarOutputStream jarOutputStream) throws IOException {
            if (this.verbose > 0) {
                UnpackerImpl.this.props.list(System.out);
            }
            int i = 1;
            while (true) {
                this.unpackSegment(bufferedInputStream, jarOutputStream);
                if (!Utils.isPackMagic(Utils.readMagic(bufferedInputStream))) {
                    break;
                }
                if (this.verbose > 0) {
                    Utils.log.info("Finished segment #" + i);
                }
                ++i;
            }
        }
        
        private void unpackSegment(InputStream inputStream, JarOutputStream out) throws IOException {
            UnpackerImpl.this.props.setProperty("unpack.progress", "0");
            new PackageReader(this.pkg, inputStream).read();
            if (UnpackerImpl.this.props.getBoolean("unpack.strip.debug")) {
                this.pkg.stripAttributeKind("Debug");
            }
            if (UnpackerImpl.this.props.getBoolean("unpack.strip.compile")) {
                this.pkg.stripAttributeKind("Compile");
            }
            UnpackerImpl.this.props.setProperty("unpack.progress", "50");
            this.pkg.ensureAllClassFiles();
            HashSet set = new HashSet(this.pkg.getClasses());
            for (Package.File file : this.pkg.getFiles()) {
                JarEntry ze = new JarEntry(Utils.getJarEntryName(file.nameString));
                int n = this.keepDeflateHint ? ((file.options & 0x1) != 0x0 || (this.pkg.default_options & 0x20) != 0x0) : this.deflateHint;
                int n2 = n == 0;
                if (!(n2 == 0)) {
                    this.crc.reset();
                }
                this.bufOut.reset();
                if (file.isClassStub()) {
                    Package.Class stubClass = file.getStubClass();
                    assert stubClass != null;
                    new ClassWriter(stubClass, n2 ? this.crcOut : this.bufOut).write();
                    set.remove(stubClass);
                }
                else {
                    file.writeTo(n2 ? this.crcOut : this.bufOut);
                }
                ze.setMethod(n ? 8 : 0);
                if (!(n2 == 0)) {
                    if (this.verbose > 0) {
                        Utils.log.info("stored size=" + this.bufOut.size() + " and crc=" + this.crc.getValue());
                    }
                    ze.setMethod(0);
                    ze.setSize((long)this.bufOut.size());
                    ze.setCrc(this.crc.getValue());
                }
                if (this.keepModtime) {
                    ze.setTime((long)file.modtime);
                    ze.setTime((long)file.modtime * 1000L);
                }
                else {
                    ze.setTime((long)this.modtime * 1000L);
                }
                out.putNextEntry(ze);
                this.bufOut.writeTo(out);
                out.closeEntry();
                if (this.verbose > 0) {
                    Utils.log.info("Writing " + Utils.zeString(ze));
                }
                else {
                    continue;
                }
            }
            assert !(!set.isEmpty());
            UnpackerImpl.this.props.setProperty("unpack.progress", "100");
            this.pkg.reset();
            return;
        }
    }
}
